#include <SDL.h>
#include <GL/gl.h>
#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include "sound.h"
#include "config.h"
#include "globals.h"
#include "menu.h"
#include "partida.h"
#include "textoogl.h"
#include "xml_glparchis.h"

#define BUFFER_LENGTH 64

//Variables
Partida *p;
Sound *s;
xml_glParchis *xml;
Menu *menu;
SDL_Surface *screen;
int frames=0; //Contador de frames
int fps=0;
int temps1=0;
int temps2=0;
bool done=false;
bool showfullscreen=false;
extern char **environ;
bool mostrar_fps=false;


void RenderScene( void );
static void iniciar_objetos();
void ChangeSize(int w, int h);
void SetupRC(int,int);
void ProcessEvent(SDL_Event *event);
void ProcessKeys(SDL_keysym *keysym);
void ProcessSelection(int xPos, int yPos, bool rightmousebutton);
void Salir();
void muestra_fps(int x, int y);

void Salir(){
  s->close_sound();
  TTF_Quit();
  Mix_CloseAudio();
  SDL_Quit();
}


void ProcessEvent(SDL_Event *event){
 while ( SDL_PollEvent(event) )
 {
  switch(event->type){

    case SDL_QUIT:
      done=true;
      break;
    case SDL_KEYDOWN:
      ProcessKeys(&event->key.keysym);
      break;
    case SDL_MOUSEBUTTONDOWN:
        if (event->button.button==SDL_BUTTON_LEFT){
          ProcessSelection(event->button.x, event->button.y, false);
        }else{          ProcessSelection(event->button.x, event->button.y, true);
        }
      break;
    default:
      break;
  }
 }
}

void ProcessSelection(int xPos, int yPos, bool rightmousebutton){
  GLfloat fAspect;
  static GLuint selectBuff[BUFFER_LENGTH];
  GLint hits, viewport[4];
  glSelectBuffer(BUFFER_LENGTH,selectBuff);
  glGetIntegerv(GL_VIEWPORT, viewport);
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glRenderMode(GL_SELECT);
  glLoadIdentity();
  gluPickMatrix(xPos,viewport[3] -yPos,1,1, viewport);
  fAspect=(float)viewport[2]/(float)viewport[3];
  gluPerspective(45.0f,fAspect,1.0,425.0);
  RenderScene();
  hits=glRenderMode(GL_RENDER);
  if (menu->visible==true){
    menu->ProcessObjeto(selectBuff,hits);
  } else {
    p->ProcessObjeto(selectBuff,hits,rightmousebutton);
  }
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
}


void ProcessKeys(SDL_keysym *keysym){
//   printf(gettext("Tecla pulsada %d\n"),keysym->sym);
  p->KeyPressFunc(keysym->sym,0,0);
  switch(keysym->sym){
    case SDLK_f://f
        SDL_WM_ToggleFullScreen(screen);
        break;
    case SDLK_l://l
        xml->load("/tmp/glparchis.sav");
        break;
    case SDLK_s://s
        xml->save("/tmp/glparchis.sav");
        break;
    case SDLK_F1://F1 
        mostrar_fps=!mostrar_fps;
        break;
  }
}


void SetupRC(int width, int height)
{
  float ratio = width/height;

  glEnable(GL_DEPTH_TEST);
  glFrontFace(GL_CCW);

  GLfloat light_ambient[] =  {0.4, 0.4, 0.4, 1.0};
  GLfloat light_diffuse[] =  {0.3, 0.3, 0.3, 1.0};
  GLfloat light_specular[] =  {1, 1, 1, 1.0};
  GLfloat light_position[] =  {5.0, 5.0, 5.0, 0.0};


  glEnable(GL_LIGHTING);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glEnable(GL_LIGHT0);

  glEnable(GL_COLOR_MATERIAL);
  glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
  glShadeModel (GL_SMOOTH);
  glClearColor(0.0f,0.0f,0.0f,1.0f);


}

static void iniciar_objetos(){
  p=new Partida();
  menu=new Menu(screen);
  xml=new xml_glParchis(p);
  s=new Sound();

}

int main(int argc, char* argv[])
{
    char current[1000];
    char *version=VERSION;
    getcwd(current,1000);
    char configprefix[1000]; 
    char domain[1000];  
    char *hola;
    sprintf(configprefix,"%s",PREFIX);
    if (strcmp(configprefix,"NONE")==0){
      sprintf(configprefix,"/usr/local");
    }
    sprintf(domain,"%s/share/", configprefix);
    printf("Domain: %s\n",domain);


    hola=bindtextdomain(GETTEXT_PACKAGE,domain);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);

    printf("Salida bindomain %s GP%s\n",hola,GETTEXT_PACKAGE);					
    printf(gettext("Directorio actual : %s\n"), current);
    printf(gettext("Version: %s\n"),version);
    printf(gettext("spanish\n"));
    int width=1024;
    int height=768;

  SDL_Event event;
   s->init_sound();
  if (TTF_Init()==-1){
    printf("Error al iniciar las fuentes\n");
  }

  screen = SDL_SetVideoMode(width,height,16, SDL_OPENGL | SDL_RESIZABLE | SDL_SWSURFACE | SDL_FULLSCREEN);
  if(!screen){
    printf(gettext("No se pudo iniciar la pantalla: %s\n"), SDL_GetError());
    Salir();
  }

  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
  SDL_WM_SetCaption("GNU GLParchis 2006",NULL);
  ChangeSize( width,height);
  iniciar_objetos();
  SetupRC(width,height);
  while (done==false) {
    RenderScene( );
    ProcessEvent(&event);
  }
  SDL_FreeSurface(screen);
  SDL_Quit();
  return 0;
}

void ChangeSize(int w, int h){
    GLfloat fAspect;
    if(h == 0)
        h = 1;

    glViewport(0, 0, w, h);

    fAspect = (GLfloat)w / (GLfloat)h;

    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Set the clipping volume
    gluPerspective(45.0f, fAspect, 1.0f, 425.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void RenderScene(void){


 
  if(p->salir==true){
    menu->visible=true;
    menu->play_selected=false;
    p->salir=false;
    menu->pulsada_nueva_seleccion_jugadores=true;
  }

  if(menu->salir==true)
    Salir();

  if (menu->visible==true){
    menu->RenderScene();
    if(mostrar_fps==true) muestra_fps(-10,-4);
    menu->EndRenderScene();
    return;
  }

  if (menu->pulsada_nueva_seleccion_jugadores){
    xml->inicio_partida(menu->tipojugador);
    xml->load("/tmp/glparchis_inicio.sav");
    menu->pulsada_nueva_seleccion_jugadores=false;
    return;
  }


  if (menu->play_selected){
    p->estado_eventos=1;
    menu->play_selected=false; //Sería mejor destruir el menu no queda bien
  }

  if (p->visible==true){
    p->RenderScene();
    if(mostrar_fps==true) muestra_fps(-8,-2);
    p->EndRenderScene();
    return;

  }

}


/*!
    \fn muestra_fps()
 */
void muestra_fps(int x, int y)
{
  frames++;
  temps2=SDL_GetTicks();
  if(((temps2-temps1)/1000)>=1){
//     printf("Frames: %d\n",int(frames)); 
    fps=frames;
    frames=0; 
    temps1=SDL_GetTicks();
  }

  glPushMatrix();
  glColor4f(0,1,0,1);
    glRasterPos2f(x,y);
    char str[1000];
    sprintf(str,"Render: %d fps",fps);
    TextoOGL a; a.drawstring(str);
  glPopMatrix();

}

